
require 'amrita2/core'

module Amrita2

  module Core
    module Runtime
      def get_amrita2_debug_data
        Thread::current[:amrita2_debug_data] 
      end

      def set_amrita2_debug_data(val)
        Thread::current[:amrita2_debug_data] = val
      end
    end

    class DynamicElementSpec
      def create_dummy_template_src(cg)
        cg.code("extend Amrita2::Core::Runtime")
        cg.define_method(sym, "val=nil", "attrs={}") do
          cg.if_("attrs.size > 0") do
            cg.code("get_amrita2_debug_data[:#{sym}_attr] = attrs")
          end
          cg.if_("block_given?") do
            cg.code("save = get_amrita2_debug_data")
            cg.code("get_amrita2_debug_data[:#{output_stream}] ||= []")
            cg.code("d = {}")
            cg.code("set_amrita2_debug_data(d)")
            cg.code("yield(#{module_name})")
            cg.code("set_amrita2_debug_data(save)")
            cg.code("get_amrita2_debug_data[:#{output_stream}] << d")
            cg.else_ do
              cg.code("get_amrita2_debug_data[:#{output_stream}] ||= []")
              cg.code("get_amrita2_debug_data[:#{output_stream}] << val")
            end
          end
        end
        cg.code("module_function :#{sym}")
        cg.code("extend Amrita2::Core::Runtime")
        cg.define_module(module_name) do
          children.each do |c|
            c.create_dummy_template_src(cg)
          end
        end
      end
    end

    class RootSpec
      def create_dummy_template
        mod = Module.new
        cg = CodeGenerator.new
        create_dummy_template_src(cg)
        src = cg.results.join("\n")
        puts src if $DEBUG
        mod.module_eval src
        mod.module_eval do
          def expand_template(out=nil, &block)
            set_amrita2_debug_data({})
            yield(self)
            @hash = flatten_hash(get_amrita2_debug_data)
          end

          def flatten_hash(h)
            return h unless h.kind_of?(Hash)
            h.keys.each do |k|
              val = h[k]
              case val
              when Array
                if val.size == 1
                  h[k] = flatten_hash(val[0])
                else
                  h[k] = val.collect do |v|
                    flatten_hash(v)
                  end
                end
              when Hash
                h[k] = flatten_hash(val)
              end
            end
            h
          end
          module_function :expand_template, :flatten_hash

          def self.[](arg)
            @hash[arg]
          end
        end
        mod
      end

      def create_dummy_template_src(cg)
        children.each do |c|
          c.create_dummy_template_src(cg)
        end
      end
    end
  end
end

