
require "rexml/document"

module REXML
  class Element
    attr_accessor :amrita_dynamic_element

    def amrita_expand_template(out)
      if amrita_dynamic_element
        out << "!!!!!!!!!!!"
        out << "<#{name}>"
        out << amrita_dynamic_element.sym
        out << "!!!!!!!!!!!"
        out << amrita_dynamic_element.value
        out << "!!!!!!!!!!!"
        out << "\n"
      else
        out << "<#{name}>"
      end
      elements.each do |e|
        e.amrita_expand_template(out)
      end
      out << "</#{name}>"
    end
  end
end

module Amrita2

  class DynamicElementSpec
    attr_reader :sym, :params, :children
    def initialize(sym, params={}, &block)
      @sym, @params, @children = sym, params, []
      if block_given?
        self.instance_eval(block)
      end
    end
  end

  class DynamicElement
    attr_reader :sym
    attr_accessor :value

    def initialize(sym)
      @sym = sym
    end
  end

  module TemplateFactory
    def dyn_elem(sym, params={})
      unless self.const_defined?(:DynElems)
        self.const_set(:DynElems, [])
      end
      self.const_get(:DynElems) << DynamicElementSpec.new(sym, params)
    end

    def create_template_class(t)
      doc = REXML::Document.new t

      d =  []
      self::DynElems.each { |e| d.push(DynamicElement.new(e.sym)) }

      dh = {}
      d.each do |e|
        dh[e.sym] = e
      end
      match_dyn_elems(doc.root, d)

      c = Class.new
      c.const_set(:DynElemsHash, dh)
      p dh
      c.module_eval do
        def output(out)
          out.puts "...."
          p self.class::Factory
          p self.class::Factory::DynElems
          yield(self)
          self.class::Doc.root.amrita_expand_template(out)
          puts ""
        end
      end

      self::DynElems.each do |m|
        s = <<-END
          def #{m.sym}(s)
            p s
            print DynElemsHash.inspect
            puts ""
            DynElemsHash[:#{m.sym}].value = s
          end
        END
        c.module_eval s
      end
      c.const_set(:Doc, doc)
      c.const_set(:Factory, self)
      c
    end

    def match_dyn_elems(e, d)
      return unless d.size > 0
      if e.attributes["id"] == d.first.sym.to_s
        e.amrita_dynamic_element = d.shift
      end
      e.elements.each do |c|
        match_dyn_elems(c, d)
      end
    end
  end
end

if __FILE__ == $0
  TEMPLATE_TEXT = <<-END
  <html>
    <body>
      <h1 id="title">title</h1>
      <p id="text">text</p>
    </body>
  </html>
  END

  class Hello
    extend Amrita2::TemplateFactory

    dyn_elem(:title)
    dyn_elem(:text)
  end

  T = Hello.create_template_class(TEMPLATE_TEXT)

  t = T.new

  t.output(STDOUT) do |e|
    e.title("hello amrita2")
    e.text <<-END
    Generated by amrita2
    END
  end

end
