
PRJDIR=File::expand_path("~/amrita2")
$:.unshift PRJDIR+'/lib'

require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/template'
require 'rexml/document'

module Amrita2Test

  class TestDefaultSpec < Test::Unit::TestCase
    include Amrita2
    
    def test_default_spec
      doc = REXML::Document.new "<a id='aaa'><b id='bbb'></b></a>"
      root = doc.root
      spec = Amrita2::make_default_spec(root)
      assert_equal(nil, spec.sym)
      assert_equal(1, spec.children.size)
      assert_equal(:aaa, spec.children[0].sym)
      assert_equal(1, spec.children[0].children.size)
      assert_equal(:bbb, spec.children[0].children[0].sym)
      assert_equal(0, spec.children[0].children[0].children.size)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["b"].amrita_spec)
      assert(:aaa, root.elements["b"].amrita_spec.sym)

      out = ""
      t.expand_template(out) do |m|
        m.aaa do |m2|
          m2.bbb('yyy')
        end
      end

      assert_equal("<a><b>yyy</b></a>", out)
    end

    def test_default_spec2
      doc = REXML::Document.new <<-END
<html>
   <body>
      <h1 id='title'>title will be inserted here</h1>
      <p id='body'>body text will be inserted here</p>
   </body>
</html>
END

      root = doc.root
      spec = Amrita2::make_default_spec(root)
      assert_equal(nil, spec.sym)
      assert_equal(2, spec.children.size)
      assert_equal(:title, spec.children[0].sym)
      assert_equal(0, spec.children[0].children.size)
      assert_equal(:body, spec.children[1].sym)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["body/h1"].amrita_spec)
      assert(:aaa, root.elements["body/h1"].amrita_spec.sym)

      out = ""
      t.expand_template(out) do |m|
        m.title('ttt')
        m.body('bbb')
      end

      assert_equal("<html>\n   <body>\n      <h1>ttt</h1>\n      <p>bbb</p>\n   </body>\n</html>", out)
    end
  end

  class TestDefaultExpand < Test::Unit::TestCase
    include Amrita2

    def test_default_spec
      doc = REXML::Document.new "<a id='aaa'><b id='bbb'></b></a>"
      root = doc.root
      spec = Amrita2::make_default_spec(root)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["b"].amrita_spec)
      assert(:aaa, root.elements["b"].amrita_spec.sym)

      data = {
        :aaa =>  {
          :bbb => "yyy"
        }
      }
      out = ""
      Amrita2::default_expand(t, out, data)
      assert_equal("<a><b>yyy</b></a>", out)

      data = {
        :aaa =>  {
          :bbb => [1,2,3]
        }
      }
      out = ""
      Amrita2::default_expand(t, out, data)
      assert_equal("<a><b>1</b><b>2</b><b>3</b></a>", out)
    end
  end

  class TestTemplate < Test::Unit::TestCase
    include Amrita2

    def test_template
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb('xxx')
        end
      end
      assert_equal("<a><b>xxx</b></a>", out)
    end
  end
end

if __FILE__ == $0
  include Amrita2Test
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestTemplate.suite
    suite << TestDefaultExpand.suite
    suite << TestDefaultSpec.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
